@@ -23,11 +23,19 @@ int ind_2(int l, int j)
 	return (j-2)*(j-1)/2 + l;
 #else
     /* from Li Long -- optimizes particularly well on Itanium2 */
+
+    /* max_m check by MMächler: max_m  is the largest integer m
+     * ----- such that (m-2)*(m-1) < MAX_INT : */
+#define max_m 46342
+
     int result = 0;
-    int m = l>j ? l : j;
-    int n = l>j ? j : l;
-    if (l != j)
-	result = (m-2)*(m-1)/2 + n;
+    if (l != j) {
+	int m = l>j ? l : j;
+	int n = l>j ? j : l;
+	result = (m <= max_m)
+	    ? (m-2)*(m-1)/2 + n
+	    : (int) (((double) m-2)*(m-1)/2 + n);
+    }
     return result;
 #endif
 }
